﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Telerik.Windows;
using Telerik.Windows.Controls.DragDrop;
using System.Collections.Generic;

namespace DragAndDropExample
{
	public class DraggableListBox :  System.Windows.Controls.ListBox
	{
		Brush listBoxDefaultBrush;
		Brush listBoxDragPossible = new SolidColorBrush(Colors.Orange);

		public DraggableListBox()
		{
			RadDragAndDropManager.SetAllowDrop(this, true);

			this.DragQuery += new EventHandler<DragDropQueryEventArgs>(OnDragQuery);
			this.DragInfo += new EventHandler<DragDropEventArgs>(OnDragInfo);
			this.DropQuery += new EventHandler<DragDropQueryEventArgs>(OnDropQuery);
			this.DropInfo += new EventHandler<DragDropEventArgs>(OnDropInfo);
		}


		public event EventHandler<DragDropQueryEventArgs> DragQuery
		{
			add
			{
				this.AddHandler(RadDragAndDropManager.DragQueryEvent, value);
			}
			remove
			{
				this.RemoveHandler(RadDragAndDropManager.DragQueryEvent, value);
			}
		}

		public event EventHandler<DragDropQueryEventArgs> DropQuery
		{
			add
			{
				this.AddHandler(RadDragAndDropManager.DropQueryEvent, value);
			}
			remove
			{
				this.RemoveHandler(RadDragAndDropManager.DropQueryEvent, value);
			}
		}

		public event EventHandler<DragDropEventArgs> DragInfo
		{
			add
			{
				this.AddHandler(RadDragAndDropManager.DragInfoEvent, value);
			}
			remove
			{
				this.RemoveHandler(RadDragAndDropManager.DragInfoEvent, value);
			}
		}

		public event EventHandler<DragDropEventArgs> DropInfo
		{
			add
			{
				this.AddHandler(RadDragAndDropManager.DropInfoEvent, value);
			}
			remove
			{
				this.RemoveHandler(RadDragAndDropManager.DropInfoEvent, value);
			}
		}

		public override void OnApplyTemplate()
		{
			base.OnApplyTemplate();
			listBoxDefaultBrush = BorderBrush;
		}

		void OnDropInfo(object sender, DragDropEventArgs e)
		{
			DraggableListBox box = sender as DraggableListBox;
			IList<ApplicationInfo> itemsSource = box.ItemsSource as IList<ApplicationInfo>;
			ApplicationInfo payload = e.Options.Payload as ApplicationInfo;

			if (e.Options.Status == DragStatus.DropPossible)
			{
				box.BorderBrush = listBoxDragPossible;
			}
			else
			{
				box.BorderBrush = listBoxDefaultBrush;
			}

			if (e.Options.Status == DragStatus.DropComplete)
			{
				if (!itemsSource.Contains(payload))
				{
					itemsSource.Add(payload);
				}
			}
		}

		void OnDropQuery(object sender, DragDropQueryEventArgs e)
		{
			DraggableListBox box = sender as DraggableListBox;
			IList<ApplicationInfo> itemsSource = box.ItemsSource as IList<ApplicationInfo>;
			ApplicationInfo payload = e.Options.Payload as ApplicationInfo;

			e.QueryResult = payload != null && !itemsSource.Contains(payload);
		}

		void OnDragInfo(object sender, DragDropEventArgs e)
		{
			DraggableListBox box = sender as DraggableListBox;
			IList<ApplicationInfo> itemsSource = box.ItemsSource as IList<ApplicationInfo>;
			ApplicationInfo payload = e.Options.Payload as ApplicationInfo;

			if (e.Options.Status == DragStatus.DragComplete)
			{
				if (payload != null && itemsSource.Contains(payload))
				{
					itemsSource.Remove(payload);
				}
			}
		}

		protected virtual void OnDragQuery(object sender, DragDropQueryEventArgs e)
		{
			DraggableListBox box = sender as DraggableListBox;

			e.QueryResult = true;
			if (e.Options.Status == DragStatus.DragQuery)
			{
				//e.Options.SourceCueHost = null;
				e.Options.Payload = box.SelectedItem;
				ContentControl cue = new ContentControl();
				cue.ContentTemplate = this.Resources["ApplicationDragTemplate"] as DataTemplate;
				cue.Content = box.SelectedItem;
				e.Options.DragCue = cue;
				e.Options.ArrowCue = RadDragAndDropManager.GenerateArrowCue();
			}
		}

		protected override void PrepareContainerForItemOverride(DependencyObject element, object item)
		{
			base.PrepareContainerForItemOverride(element, item);
			RadDragAndDropManager.SetAllowDrag(element, true);
			RadDragAndDropManager.SetAllowDrop(element, true);
		}

		protected override void ClearContainerForItemOverride(DependencyObject element, object item)
		{
			base.ClearContainerForItemOverride(element, item);
			RadDragAndDropManager.SetAllowDrag(element, false);
			RadDragAndDropManager.SetAllowDrop(element, false);
		}
	}
}
