﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Telerik.Windows.Controls.DragDrop;
using System.Collections.ObjectModel;
using Telerik.Windows.Controls.Primitives;


namespace DragAndDropExample
{
	public partial class MainPage : UserControl
	{
		public static ObservableCollection<ApplicationInfo> GenerateApplicationInfos()
		{
			return new ObservableCollection<ApplicationInfo>()
            {
                new ApplicationInfo()
                {
                    Name = "Large Collider",
                    Author = "C.E.R.N.",
                    IconPath = @"/DragAndDropExample;component/Images/LargeIcons/Atom.png"
                },
                new ApplicationInfo()
                {
                    Name = "Paintbrush",
                    Author = "Imagine Inc.",
                    IconPath = @"/DragAndDropExample;component/Images/LargeIcons/Brush.png"
                },
                new ApplicationInfo()
                {
                    Name = "Lively Calendar",
                    Author = "Control AG",
                    IconPath = @"/DragAndDropExample;component/Images/LargeIcons/CalendarEvents.png"
                },
                new ApplicationInfo()
                {
                    Name = "Fire Burning ROM",
                    Author = "The CD Factory",
                    IconPath = @"/DragAndDropExample;component/Images/LargeIcons/CDBurn.png"
                },
                new ApplicationInfo()
                {
                    Name = "Fav Explorer",
                    Author = "Star Factory",
                    IconPath = @"/DragAndDropExample;component/Images/LargeIcons/favorites.png"
                },
                new ApplicationInfo()
                {
                    Name = "IE Fox",
                    Author = "Open Org",
                    IconPath = @"/DragAndDropExample;component/Images/LargeIcons/Connected.png"
                },
                new ApplicationInfo()
                {
                    Name = "Charting",
                    Author = "AA-AZ inc",
                    IconPath = @"/DragAndDropExample;component/Images/LargeIcons/ChartDot.png"
                },
                new ApplicationInfo()
                {
                    Name = "SuperPlay",
                    Author = "EB Games",
                    IconPath = @"/DragAndDropExample;component/Images/LargeIcons/Games.png"
                }
            };
		}

		ObservableCollection<ApplicationInfo> allApplications = GenerateApplicationInfos();
		ObservableCollection<ApplicationInfo> myApplications = new ObservableCollection<ApplicationInfo>();
		Brush listBoxDragPossible = new SolidColorBrush(Colors.Orange);

		public MainPage()
		{
			InitializeComponent();
			myApplications = new ObservableCollection<ApplicationInfo>();
			allApplicationsBox.ItemsSource = allApplications;
			myApplicationsBox.ItemsSource = myApplications;

			RadDragAndDropManager.AddDragQueryHandler(this, OnDragQuery);
			RadDragAndDropManager.AddDragInfoHandler(this, OnDragInfo);
			RadDragAndDropManager.AddDropQueryHandler(this, OnDropQuery);
			RadDragAndDropManager.AddDropInfoHandler(this, OnDropInfo);
		}

	

		private void OnDropInfo(object sender, DragDropEventArgs e)
		{
			ItemsControl box = e.Options.Destination as ItemsControl;
			IList<ApplicationInfo> itemsSource = box.ItemsSource as IList<ApplicationInfo>;
			ApplicationInfo payload = e.Options.Payload as ApplicationInfo;

			if (e.Options.Status == DragStatus.DropPossible)
			{
				box.BorderBrush = listBoxDragPossible;
			}
			else
			{
				box.BorderBrush = new SolidColorBrush(Colors.Gray);
			}

			if (e.Options.Status == DragStatus.DropComplete)
			{
				if (!itemsSource.Contains(payload))
				{
					itemsSource.Add(payload);
				}
			}
		}

		void OnDropQuery(object sender, DragDropQueryEventArgs e)
		{
			ItemsControl box = e.Options.Destination as ItemsControl;
			IList<ApplicationInfo> itemsSource = box.ItemsSource as IList<ApplicationInfo>;
			ApplicationInfo payload = e.Options.Payload as ApplicationInfo;

			e.QueryResult = payload != null && !itemsSource.Contains(payload);
		}

		void OnDragInfo(object sender, DragDropEventArgs e)
		{
			ListBoxItem listBoxItem = e.Options.Source as ListBoxItem;
			ListBox box = ItemsControl.ItemsControlFromItemContainer(listBoxItem) as ListBox;
			IList<ApplicationInfo> itemsSource = box.ItemsSource as IList<ApplicationInfo>;
			ApplicationInfo payload = e.Options.Payload as ApplicationInfo;

			if (e.Options.Status == DragStatus.DragComplete)
			{
				if (payload != null && itemsSource.Contains(payload))
				{
					itemsSource.Remove(payload);
				}
			}
		}

		protected virtual void OnDragQuery(object sender, DragDropQueryEventArgs e)
		{
			ListBoxItem listBoxItem = e.Options.Source as ListBoxItem;
			ListBox box = ItemsControl.ItemsControlFromItemContainer(listBoxItem) as ListBox;

			if (e.Options.Status == DragStatus.DragQuery && box != null)
			{
				e.Options.Payload = box.SelectedItem;
				ContentControl cue = new ContentControl();
				cue.ContentTemplate = this.Resources["ApplicationDragTemplate"] as DataTemplate;
				cue.Content = box.SelectedItem;
				e.Options.DragCue = cue;
				e.Options.ArrowCue = RadDragAndDropManager.GenerateArrowCue();
			}

			e.QueryResult = true;
		}
	}


}
